#define LED_ZUSTAND 10
#define LED_TASTER 11
#define LED_SPERRE 8
#define TASTER 9
#define EMAGNET 5
#define MESSUNG_IN 12                  //output through a 150 ohm resistor to the LC circuit
#define MESSUNG_OUT 13                 //input from the comparator output

bool zustand = 0;                      //0: Tuerverriegelung geschlossen
bool bestromung = 0;                   //0: Elektromagnet nicht bestromt
bool taster = 0;                       //0: Taster nicht betätigt
bool sperre = 0;                       //1: Bestromen verboten
bool taster_alt;            
int zeit_ein = 0;                      //Dauer der Taster-Betätigung ca. in 100 ms
int zeit_aus = 0;                      //Dauer der Taster-Nicht-betätigung ca. in 100 ms
int i = 100;                           //Zaehlvariable: Startwert restlos teilbar durch 10, damit Messung sofort anläuft
float addition, teiler;                //Hilfsvariablen
float pulse, frequency, capacitance, inductance, inductance_mH, induktivitaet_mH;
float max_induktivitaet_zu = 2.6;
float min_induktivitaet_offen = 2.7;

void setup()
{
  pinMode(LED_ZUSTAND, OUTPUT);
  pinMode(LED_SPERRE, OUTPUT);
  pinMode(LED_TASTER, OUTPUT);
  pinMode(TASTER, INPUT);
  pinMode(EMAGNET, OUTPUT);
  pinMode(MESSUNG_OUT, INPUT);
  pinMode(MESSUNG_IN, OUTPUT);
  Serial.begin(115200);
  Serial.println("----INDUKTIVITAETSMESSUNG----");
  delay(200);                              //in miliseconds(alternativ: 2500ms)
}

void loop()
{
  taster = digitalRead(TASTER);
  
  //---------------------------------------------Sicherheitsmaßnahmen---------------------------------------------
  if (taster == 1 && taster_alt == 0)       //steigende Flanke
  {
    zeit_aus = 0;
  }
  if (taster == 0 && taster_alt == 1)       //fallende Flanke
  {
    zeit_ein = 0;
  }
  
  if (zeit_ein >= 200)                       //Elektromagnet maximal für ca. 20 s ansteuern,...
  {
    sperre = 1;
    digitalWrite(LED_SPERRE, HIGH);
    Serial.println("Sperre aktiv");
  }
  if (zeit_aus >= 600)                       //...sonst ca. 1 min warten (abkühlen lassen)
  {
    sperre = 0;
    digitalWrite(LED_SPERRE, LOW);
  }

  
  if (sperre == 0 && zeit_ein > 0)           //Wenn Taster betätigt, Zeit anzeigen, wie lange noch bestromt werden darf
  {
    Serial.print("Elektromagnet darf noch fuer ");
    Serial.print((200-zeit_ein)/10);
    Serial.println(" s bestromt werden.");
  }
  if (sperre == 1 && zeit_aus > 0)           //Wenn nach zu langer Bestromung der Taster losgelassen wurde, Wartezeit anzeigen
  {
    Serial.print("Taster darf noch ");
    Serial.print((600-zeit_aus)/10);
    Serial.println(" s nicht gedrueckt werden.");
  }

  //---------------------------------------------Magnetansteuerung---------------------------------------------
  if (taster == 1 && sperre == 0) 
  {
    bestromung = 1;
    digitalWrite(EMAGNET, HIGH);
    digitalWrite(LED_TASTER, HIGH);
    Serial.println("Elektromagnet wird bestromt!");
  }
  else
  {
    bestromung = 0;
    digitalWrite(EMAGNET, LOW);
    digitalWrite(LED_TASTER, LOW);
    Serial.println("Elektromagnet wird nicht bestromt.");
  }
  delay(10);

  //---------------------------------------------Induktivitätsmessung---------------------------------------------
  if(i%10 == 0 && bestromung == 0)          //10: nur alle 1 Sekunden und nur, wenn Elektromagnet gerade nicht bestromt wird, messen
                                            //Von 10 auf 0 herunterzählen, statt Konstrukt mit MODULO, nicht möglich, da, wenn bestromung = 1, bis 20 gezählt werden muss (dann insgesamt 2s Messabstand)
  {              
    i = 0;
    for(int j=0; j<=10; j++)                 //10 mal messen
    {

        digitalWrite(MESSUNG_IN, HIGH);
        delay(5);                           //Warten bis Induktivität aufgeladen
        digitalWrite(MESSUNG_IN,LOW);
        delayMicroseconds(100);             //kurz Warten
        pulse = pulseIn(MESSUNG_OUT,HIGH,5000);        //0 = timeout: pulseIn(pin, value, timeout)
        if(pulse > 0.1)                     //Wenn kein timeout
        {
          capacitance = 1.E-6;              //1µF (alternativ: 200nF = 2.E-7)
          frequency = 1.E6/(2*pulse);
          inductance = 1/(capacitance*frequency*frequency*4*3.14159*3.14159);
          inductance_mH = inductance*1.E3;
//          Serial.print("High-Pegel fuer us: ");
//          Serial.print(pulse);
//          Serial.print("   Frequenz in Hz: ");
//          Serial.print( frequency );
//          Serial.print("   momenteane Induktivität in mH:");
//          Serial.println(inductance_mH);
          addition += inductance_mH;
          teiler ++;
        }
        
        else                                  //Wenn timeout
        {
          Serial.println("\t\t\t\t!Fehler_1: Timeout!");
        }

        delay(20);                           //z.B. oder 100
    }

    induktivitaet_mH = addition/teiler;      //Mittelwert aus 10 Messungen bilden
    addition = 0;
    teiler = 0;
    Serial.print("\tInduktivitaet: ");
    Serial.print(induktivitaet_mH);
    Serial.println(" mH");

    if(induktivitaet_mH < max_induktivitaet_zu)
    {
      zustand = 0;                           //Magnetischer Kreis geschlossen
      digitalWrite(LED_ZUSTAND, LOW);
      Serial.println("\tTuere ist geschlossen.");
    }
    else if (induktivitaet_mH > min_induktivitaet_offen)
    {
      zustand = 1;                           //Magnetischer Kreis offen
      digitalWrite(LED_ZUSTAND, HIGH);
      Serial.println("\t\tTuere ist geöffnet!");
    }
    else
    {
      Serial.println("\t\t\t\t!!Fehler_2: ungültiger Induktivitätswert!!");
    }
  }

  //---------------------------------------------Hilfsvariablen setzen---------------------------------------------
  taster_alt = taster;                      //Merken für Flankenauswertung
  if (taster == 1)
  {
    zeit_ein++;
  }
  if (taster == 0)
  {
    zeit_aus++;
  }
  i++;
  delay(100);                                //100 Millisekunde = 0,1 Sekunde warten (bis zur naechsten Abfrage des Tasters)
}
